/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.compiler.wrapper.dista.C45;
import java.io.*;

/**
 * @author Alessandro Serra, Timm Euler
 * @version $Id: NumericCond.java,v 1.4 2006/04/11 14:10:09 euler Exp $
 */
public class NumericCond extends Cond {
	public static final int le_op=1;
	public static final int gr_op=2;

	protected double m_value;
	protected int m_op;

	public NumericCond() {
		super();
	}

	final public double getValue() {
		return m_value;
	}
	final public void setValue(double v) {
		m_value=v;
	}
	final public int getOp() {
		return m_op;
	}
	final public void setOp(int op) {
		m_op=op;
	}
	final public String getOpName() {
		String r;
		switch(m_op) {
			case le_op:
				r="le";
				break;
			case gr_op:
				r="gr";
				break;
			default:
				r="unknow";
		};
		return r;
	}
	final public String getOpSymbol() {
		String r;
		switch(m_op) {
			case le_op:
				r="<=";
				break;
			case gr_op:
				r=">";
				break;
			default:
				r="unknow";
		};
		return r;
	}
	final public void setOp(String v) throws Exception {
		if(v.equals("le")) {
			m_op=le_op;
		} else if(v.equals("gr")) {
			m_op=gr_op;
		} else {
			throw new Exception("Bad operator name");
		}
	}

	public void dumpOn(PrintWriter w) {
		super.dumpOn(w);
		w.print(" "+getOpName()+" "+m_value+" ");
	}
};
/*
 * Historie
 * --------
 * 
 * $Log: NumericCond.java,v $
 * Revision 1.4  2006/04/11 14:10:09  euler
 * Updated license text.
 *
 * Revision 1.3  2006/04/06 16:31:09  euler
 * Prepended license remark.
 *
 * Revision 1.2  2006/03/30 16:07:12  scholz
 * fixed author tags for release
 *
 * Revision 1.1  2006/01/03 09:54:02  hakenjos
 * Initial version!
 *
 */
