/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.compiler.wrapper.dista.C45;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.Reader;

import edu.udo.cs.miningmart.operator.ExecutableOperator;

import edu.udo.cs.miningmart.m4.Step;

/**
 * 
 * @author Alessandro Serra, Timm Euler
 * @version $Id: RulesWrapper.java,v 1.4 2006/04/11 14:10:09 euler Exp $
 */
public class RulesWrapper extends C45Wrapper {
	protected Rules m_rules;
	protected double m_pruning_confidence;
	protected int m_min_obj;

	public RulesWrapper(Step step, ExecutableOperator exOp) throws Exception {
		super(step, exOp);
		m_pruning_confidence=0.25;
		m_min_obj=2;
	}

	public void setPruningConfidence(double v) {
		m_pruning_confidence=v;
	}
	public void setMinObj(int v) {
		m_min_obj=v;
	}

	protected void callTool() throws Exception {
		callScript("c4.5 -c "+m_pruning_confidence
				+" -m "+m_min_obj
				+" -f "+ m_file_name);
		callScript("c4.5rules -c "+m_pruning_confidence
				+" -f "+ m_file_name);
		callScript("rules2dlf -f "+ m_file_name);
	}
	protected void importResult() throws Exception {
		Reader r = new BufferedReader(
				new FileReader(m_file_name+"_rules.dlf"));
		RulesParser rp=new RulesParser(r);
		m_rules=rp.parse();
	}
	protected void saveResult() throws Exception {
		Rules2PLSql r2sql=new Rules2PLSql();
		r2sql.setMetaData(m_md);
		r2sql.setResultName(m_result_name);
		r2sql.setConnection(this.getM4Db());
		r2sql.convert(m_rules);
	}	
};
/*
 * Historie
 * --------
 * 
 * $Log: RulesWrapper.java,v $
 * Revision 1.4  2006/04/11 14:10:09  euler
 * Updated license text.
 *
 * Revision 1.3  2006/04/06 16:31:09  euler
 * Prepended license remark.
 *
 * Revision 1.2  2006/03/30 16:07:12  scholz
 * fixed author tags for release
 *
 * Revision 1.1  2006/01/03 09:54:02  hakenjos
 * Initial version!
 *
 */
