/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.m4.utils;

import edu.udo.cs.miningmart.exception.M4Exception;
import edu.udo.cs.miningmart.m4.core.Coordinates;
import edu.udo.cs.miningmart.m4.core.M4Object;
import edu.udo.cs.miningmart.m4.core.GraphicalM4Object;
;

/**
 * Helper class for standard communication between <code>GraphicalM4Object</code> 
 * and <code>Coordinates</code>.
 * 
 * @author Timm Euler
 * @version $Id: InterM4GraphicalObjectToCoordinates.java,v 1.3 2006/04/11 14:10:09 euler Exp $
 */
public class InterM4GraphicalObjectToCoordinates extends InterM4ObjectToObject{
	
	/**
	 * @param graphObject a GraphicalM4Object
	 * 
	 * @see edu.udo.cs.miningmart.m4.utils.InterM4ObjectToObject#getFirstSingleRef(M4Object)
	 */
	M4Object getFirstSingleRef(M4Object graphObject) throws M4Exception
	{	return ((GraphicalM4Object) graphObject).getCoordinates();	}

	/**
	 * @param coords a Coordinates object
	 * 
	 * @see edu.udo.cs.miningmart.m4.utils.InterM4ObjectToObject#getSecondSingleRef(M4Object)
	 */
	M4Object getSecondSingleRef(M4Object coords) throws M4Exception
	{	return (GraphicalM4Object) ((Coordinates) coords).getMyM4Object();	}

	/**
	 * @param graphObject a GraphicalM4Object
	 * @param coords a Coordinates object
	 * 
	 * @see edu.udo.cs.miningmart.m4.utils.InterM4ObjectToObject#setFirstSingleRefPrimitive(M4Object, M4Object)
	 */
	void setFirstSingleRefPrimitive(M4Object graphObject, M4Object coords)
		throws M4Exception
	{   ((GraphicalM4Object) graphObject).primitiveSetCoordinates((Coordinates) coords);   }

	/**
	 * @param coords a Coordinates object
	 * @param graphObject a GraphicalM4Object
	 * 
	 * @see edu.udo.cs.miningmart.m4.utils.InterM4ObjectToObject#setSecondSingleRefPrimitive(M4Object, M4Object)
	 */
	void setSecondSingleRefPrimitive(M4Object coords, M4Object graphObject)
		throws M4Exception
	{   ((Coordinates) coords).primitiveSetMyM4Object((GraphicalM4Object) graphObject);   }
}
/*
 * Historie
 * --------
 * 
 * $Log: InterM4GraphicalObjectToCoordinates.java,v $
 * Revision 1.3  2006/04/11 14:10:09  euler
 * Updated license text.
 *
 * Revision 1.2  2006/04/06 16:31:09  euler
 * Prepended license remark.
 *
 * Revision 1.1  2006/01/12 11:33:11  euler
 * Changed the way coordinates are stored completely.
 *
 */
