/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.m4.utils;

import edu.udo.cs.miningmart.exception.M4Exception;
import edu.udo.cs.miningmart.m4.core.ForeignKey;
import edu.udo.cs.miningmart.m4.core.M4Object;
import edu.udo.cs.miningmart.m4.core.Relation;

/**
 * Helper class for standard communication between <code>Relation</code> 
 * and <code>Key</code>, where the ToKey of the Relation is meant.
 * 
 * @author Timm Euler
 * @version $Id: InterM4RelationToToKey.java,v 1.3 2006/04/11 14:10:10 euler Exp $
 */
public class InterM4RelationToToKey extends InterM4ObjectToObject
{
	/**
	 * @param relation a Relation 
	 * 
	 * @see edu.udo.cs.miningmart.m4.utils.InterM4ObjectToObject#getFirstSingleRef(M4Object)
	 */
	M4Object getFirstSingleRef(M4Object relation) throws M4Exception
	{	return (ForeignKey) ((Relation) relation).getToKey();	}

	/**
	 * @param fkey a ForeignKey
	 * 
	 * @see edu.udo.cs.miningmart.m4.utils.InterM4ObjectToObject#getSecondSingleRef(M4Object)
	 */
	M4Object getSecondSingleRef(M4Object fkey) throws M4Exception
	{	return (Relation) ((ForeignKey) fkey).getRelation();	}

	/**
	 * @param relation a Relation 
	 * @param fkey a ForeignKey
	 * 
	 * @see edu.udo.cs.miningmart.m4.utils.InterM4ObjectToObject#setFirstSingleRefPrimitive(M4Object, M4Object)
	 */
	void setFirstSingleRefPrimitive(M4Object relation, M4Object fkey)
		throws M4Exception
	{   ((Relation) relation).primitiveSetToKey((ForeignKey) fkey);   }

	/**
	 * @param fkey a ForeignKey
	 * @param relation a Relation
	 * 
	 * @see edu.udo.cs.miningmart.m4.utils.InterM4ObjectToObject#setSecondSingleRefPrimitive(M4Object, M4Object)
	 */
	void setSecondSingleRefPrimitive(M4Object fkey, M4Object relation)
		throws M4Exception
	{   ((ForeignKey) fkey).setRelationPrimitive((Relation) relation);   }
}
/*
 * Historie
 * --------
 * 
 * $Log: InterM4RelationToToKey.java,v $
 * Revision 1.3  2006/04/11 14:10:10  euler
 * Updated license text.
 *
 * Revision 1.2  2006/04/06 16:31:10  euler
 * Prepended license remark.
 *
 * Revision 1.1  2006/01/03 09:54:03  hakenjos
 * Initial version!
 *
 */
