/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.storedProcedures;

/**
 * @author Martin Scholz
 * @version $Id: TimeValueArray.java,v 1.4 2006/04/11 14:10:16 euler Exp $
 */
public class TimeValueArray {

    final private long   startTime;
    final private double startValue;

    private long   endTime;
    private double endValue;
    private int    size;
    private double sum;

    /** Initializes the object with a first entry which must not be <i>null</i>! */
    public TimeValueArray(TimeValue t) {
	startTime  = t.getTimeAsLong();
	startValue = t.getValue();
	endTime    = startTime;
	endValue   = startValue;
	size = 1;
	sum = startValue;
    }

    public long getStartTime() {
	return startTime;
    }

    public double getStartValue() {
	return startValue;
    }

    public long getLatestTime() {
	return endTime;
    }

    public double getLatestValue() {
	return endValue;
    }

    public int getSize() {
	return size;
    }

    public double getSum() {
	return sum;
    }

    public void add(TimeValue t) {
	if (t == null)
	    return;
	size++;
	endTime  = t.getTimeAsLong();
	endValue = t.getValue();
	sum += endValue;
    }

    public double getAverageValue() {
	return (getSum() / getSize());
    }

    public double getIncreaseByTime() {
	double valueDiff = getLatestValue() - getStartValue();
	long   timeDiff  = getLatestTime()  - getStartTime();
	if (timeDiff == 0)
	    return 0;
	else return (valueDiff / timeDiff);
    }

    public double extrapolateValueFor(long time) {
	double increase = getIncreaseByTime() * (time - getLatestTime());
	return (getLatestValue() + increase);
    }

}
/*
 *$Log: TimeValueArray.java,v $
 *Revision 1.4  2006/04/11 14:10:16  euler
 *Updated license text.
 *
 *Revision 1.3  2006/04/06 16:31:16  euler
 *Prepended license remark.
 *
 *Revision 1.2  2006/03/30 16:07:15  scholz
 *fixed author tags for release
 *
 *Revision 1.1  2006/01/03 09:54:23  hakenjos
 *Initial version!
 * 
 */
